'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* global wx */

var WebSocket = function () {
  function WebSocket(uri, protocols, opts) {
    _classCallCheck(this, WebSocket);

    wx.onSocketOpen(this.handleSocketOpen.bind(this));
    wx.onSocketClose(this.handleSocketClose.bind(this));
    wx.onSocketMessage(this.handleMessage.bind(this));
    wx.onSocketError(this.handleSocketError.bind(this));
    wx.connectSocket({ url: uri });
  }

  WebSocket.prototype.handleSocketOpenLater = function handleSocketOpenLater(res) {
    var _this = this;

    if (this.handleSocketOpenLaterHandler) {
      clearTimeout(this.handleSocketOpenLaterHandler);
    }

    this.handleSocketOpenLaterHandler = setTimeout(function () {
      _this.handleSocketOpen(res);
    }, 1000);
  };

  WebSocket.prototype.handleSocketOpen = function handleSocketOpen(res) {
    if (this.onopen) {
      this.onopen(res);
    } else {
      this.handleSocketOpenLater(res);
    }
  };

  WebSocket.prototype.handleMessage = function handleMessage(res) {
    if (this.onmessage) {
      this.onmessage(res);
    } else {
      this.handleMessageLater(res);
    }
  };

  WebSocket.prototype.handleMessageLater = function handleMessageLater(res) {
    var _this2 = this;

    if (this.handleMessageLaterHandler) {
      clearTimeout(this.handleMessageLaterHandler);
    }

    this.handleMessageLaterHandler = setTimeout(function () {
      _this2.handleMessage(res);
    }, 1000);
  };

  WebSocket.prototype.handleSocketError = function handleSocketError(res) {
    if (this.onerror) {
      this.onerror(res);
    } else {
      this.handleSocketErrorLater(res);
    }
  };

  WebSocket.prototype.handleSocketErrorLater = function handleSocketErrorLater(res) {
    var _this3 = this;

    if (this.handleSocketErrorLaterHandler) {
      clearTimeout(this.handleSocketErrorLaterHandler);
    }

    this.handleSocketErrorLaterHandler = setTimeout(function () {
      _this3.handleSocketError(res);
    }, 1000);
  };

  WebSocket.prototype.handleSocketClose = function handleSocketClose(res) {
    if (this.onclose) {
      this.onclose(res);
    } else {
      this.handleSocketCloseLater(res);
    }
  };

  WebSocket.prototype.handleSocketCloseLater = function handleSocketCloseLater(res) {
    var _this4 = this;

    if (this.handleSocketCloseLaterHandler) {
      clearTimeout(this.handleSocketCloseLaterHandler);
    }

    this.handleSocketCloseLaterHandler = setTimeout(function () {
      _this4.handleSocketClose(res);
    });
  };

  WebSocket.prototype.send = function send(data) {
    wx.sendSocketMessage({
      data: data, fail: function fail(res) {
        console.log('wx web socket send failed: ' + res);
      }
    });
  };

  WebSocket.prototype.close = function close() {
    wx.closeSocket();
  };

  return WebSocket;
}();

global.WebSocket = WebSocket;

var SCClientSocket = require('socketcluster-client/lib/scclientsocket');

var scErrors = require('sc-errors');

var InvalidArgumentsError = scErrors.InvalidArgumentsError;

var _clients = {};

function getMultiplexId(options) {
  var protocolPrefix = options.secure ? 'https://' : 'http://';
  var queryString = '';
  if (options.query) {
    if (typeof options.query === 'string') {
      queryString = options.query;
    } else {
      var queryArray = [];
      var queryMap = options.query;
      for (var key in queryMap) {
        if (queryMap.hasOwnProperty(key)) {
          queryArray.push(key + '=' + queryMap[key]);
        }
      }
      if (queryArray.length) {
        queryString = '?' + queryArray.join('&');
      }
    }
  }
  var host = void 0;
  if (options.host) {
    host = options.host;
  } else {
    host = options.hostname + ':' + options.port;
  }
  return protocolPrefix + host + options.path + queryString;
}

function create() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};


  if (options.host && !options.host.match(/[^:]+:\d{2,5}/)) {
    throw new InvalidArgumentsError('The host option should include both' + ' the hostname and the port number in the format "hostname:port"');
  }

  if (options.host && options.hostname) {
    throw new InvalidArgumentsError('The host option should already include' + ' the hostname and the port number in the format "hostname:port"' + ' - Because of this, you should never use host and hostname options together');
  }

  if (options.host && options.port) {
    throw new InvalidArgumentsError('The host option should already include' + ' the hostname and the port number in the format "hostname:port"' + ' - Because of this, you should never use host and port options together');
  }

  var opts = {
    port: options.port,
    hostname: 'localhost',
    path: '/socketcluster/',
    secure: false,
    autoConnect: true,
    autoReconnect: true,
    autoSubscribeOnConnect: true,
    connectTimeout: 20000,
    ackTimeout: 10000,
    timestampRequests: false,
    timestampParam: 't',
    authEngine: null,
    authTokenName: 'socketCluster.authToken',
    binaryType: 'arraybuffer',
    multiplex: true,
    pubSubBatchDuration: null,
    cloneData: false
  };
  for (var i in options) {
    if (options.hasOwnProperty(i)) {
      opts[i] = options[i];
    }
  }
  opts.clientMap = _clients;
  opts.clientId = getMultiplexId(opts);

  if (_clients[opts.clientId]) {
    if (opts.autoConnect) {
      _clients[opts.clientId].connect();
    }
  } else {
    _clients[opts.clientId] = new SCClientSocket(opts);
  }
  return _clients[opts.clientId];
}

function destroy(socket) {
  socket.destroy();
}

module.exports.factory = {
  create: create,
  destroy: destroy,
  clients: _clients
};
module.exports.SCClientSocket = SCClientSocket;

module.exports.Emitter = require('component-emitter');

module.exports.create = function (options) {
  return create(options);
};

module.exports.connect = module.exports.create;

module.exports.destroy = function (socket) {
  return destroy(socket);
};

module.exports.clients = _clients;